------------------------------------------------------------------------------------------
-- Whammy
-- Created By:  Jaycyn of Hyjal
--
-- What is it: Heal, Decurse and more with a point and click
-- License   : Distribute as you wish, please give credit to the author, please send
--           :   bugs and suggestions instead of modifying code yourself.
------------------------------------------------------------------------------------------

local WhammyVers = "0.3.9";
local WhammyDate = "03/15/2007";
W_kUnitButtonWidth = 78; --the width of each unit button
W_kUnitButtonHeight = 20;

W_kHealthBarWidth = W_kUnitButtonWidth; --the width of the health bar on each unit button\
W_kHealthBarHeight = 3;

W_kCurseWidth = W_kUnitButtonWidth; --width and height of the cursebar
W_kCurseHeight = 5;

W_kCurseLeftSideWidth = 4;
W_kCurseRightSideWidth = 4;
W_kCurseSidesHeight = 17;

-- below are functions dealing with clicking in the unit buttons to cast spells
--   note that the default action for each button is assigned when the main frame loads.
function W_UnitButton_OnLoad()
	--W_ShowMessage("Unit_Button_Onload");
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp"); -- the buttons need to see the mouse clicks
end



--
-- Handle Unit Button Mouse actions
--
function W_UnitButton_MouseDown() --MouseDown and up just highlight the unit name when clicked. arg1 is the LeftButton, etc. unused
	local buttonName = this:GetName(); --get the name of the unit button that triggered this event
	getglobal(buttonName.."_UnitName"):SetFontObject( W_kFontNameHighlight );
end

function W_UnitButton_MouseUp() -- unhighlight the name
	local unitID = this:GetAttribute("unit"); --get the name of the unit assigned to this button
	local buttonName = this:GetName(); --get the name of the button
	local online, isDead, isAUnit;
	
	if ( unitID ) then --if this is a valid unit then
		online = UnitIsConnected( unitID ); --get the unit status assigned to this button
		isDead = W_IsDead( unitID );
		isAUnit = W_IsUnit( unitID ); -- see if this is a unit instead of a pet
		
		if ( isAUnit ) then -- this is a unit and not a pet - adjust the color of the text part of the frame
			myTextFrame = getglobal( buttonName.."_UnitName" );
			if ( online ) then
				if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
					myTextFrame:SetFontObject( W_kFontNamePlayer );
				else --someone else in the party or raid
					myTextFrame:SetFontObject( W_kFontNameOthers );
				end
			else --offline, show their text disabled
				myTextFrame:SetFontObject( W_kFontNameDisable );
			end
		end
	end
end
-- End Handle Unit Button Mouse actions



--
-- below are items dealing with the main Whammy Frame: registering for events, mouse clicks etc
--
function W_Frame_Onload(self)
	-- W_ShowMessage("Frame_Onload");

	-- Register the game events neccesary for our functionality
	self:RegisterEvent("ADDON_LOADED");
	--self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("PLAYER_AURAS_CHANGED");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MANA");
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("PLAYER_REGEN_DISABLED"); --use this to know if we enter combat so we can disable the drop downs
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	--self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	--					CHAT_MSG_SPELL_AURA_GONE_SELF
	
	
--self:RegisterEvent("PET_UI_UPDATE")
--self:RegisterEvent("PLAYER_PET_CHANGED")
--self:RegisterEvent("PET_BAR_CHANGED")
--self:RegisterEvent("UNIT_PET")
	
	
	-- set up the slash commands
	SlashCmdList["WHAMMY"] = W_Slash_Command_Handler;
	SLASH_WHAMMY1 = "/whammy";
	SLASH_WHAMMY2 = "/wham";
end

-- Handle stuff relevent to the Whammy Frame
function W_Frame_MouseDown(self) --this toggles the config frame
	--W_ShowMessage("Frame_MouseDown");
	
	if arg1 == "LeftButton" and IsShiftKeyDown() then -- drag the frame
		self:StartMoving();
	elseif arg1 == "RightButton" and IsShiftKeyDown() then --need to show a frame that lets the user configure the dropdowns etc
		W_Config_Frame_Toggle();
	end
end

-- Based upon our registered events, we want different things to happen when each event occurs.
function W_Frame_OnEvent(self, event, ...)
	--W_ShowMessage( event );
	
	if ( event == "ADDON_LOADED" and arg1 == "Whammy") then -- the event occurs after the saved vars are loaded
		W_ShowMessage("Version: "..WhammyVers.." "..WhammyDate.." loaded"); --" V0.21a 1/6/2007 Loaded");
		W_ShowMessage("    /Whammy to show/hide Whammy");
		this:UnregisterEvent("ADDON_LOADED");
		gWhammyIsLoaded = true;
		W_Handle_Addon_Loaded_Event(); --set up button frames, etc
	end
	
	if ( gWhammyIsLoaded == true ) then --only take action on these events if Whammy is loaded
		if ( event == "VARIABLES_LOADED" ) then --this event occurs after we load in any saved variables. Don't do anything here. functio TBD
			--W_ShowMessage("Variables Loaded Event");
		elseif ( event =="PLAYER_LOGIN") then
			if ( GetNumRaidMembers() == 0 ) then --not a raid
				W_Handle_Party_Members_Changed_Event(); --set the frame to the appropriate size for either player or party
			else -- its a raid, set up the frame
				W_Handle_Raid_Roster_Update_Event();
			end
			
			
		elseif ( event == "PLAYER_AURAS_CHANGED" ) then --watch for this player
			W_Handle_Aura_Event("player");
		elseif ( event == "PLAYER_REGEN_DISABLED" ) then --we have entered combat
			W_Handle_Combat( true );
		elseif ( event == "PLAYER_REGEN_ENABLED" ) then --we have left combat
			W_Handle_Combat( false );
			
			
		elseif ( event == "PARTY_MEMBERS_CHANGED" ) then
			if ( GetNumRaidMembers() == 0 ) then --we only care about this event if we are not in a raid since that will
				W_Handle_Party_Members_Changed_Event(); --    generate an event as well
			end
		elseif ( event == "RAID_ROSTER_UPDATE" ) then
			W_Handle_Raid_Roster_Update_Event();
			
			
		elseif ( event == "UNIT_AURA" and arg1 ~="target" and arg1 ~= "mouseover" and arg1 ~="focus" and arg1 ~="npc" and arg1 ~= "NPC" ) then --handle curses
			W_Handle_Aura_Event( arg1 ); --arg1 is the unitID of the unit whose buff changed
		elseif ( event == "UNIT_HEALTH" and arg1 ~= "target" and arg1 ~= "mouseover" and arg1 ~="focus"  and arg1 ~= "npc" and arg1 ~= "NPC" ) then
			if ( W_IsRaid() ) then --if we are in a raid, only allow health events for raid1-40 to pass or else it will be called twice
				if ( strfind(arg1, "raid" ) ) then --does the unitID contain the text raid, then let it pass
					W_Handle_Health_Event( arg1 ); --arg1 is player, pet, partyX, partypetX, raidX, raidpetX
				end
			else -- not in a raid, allow player, party1-4 partypet1-4 to pass
				W_Handle_Health_Event( arg1 ); --arg1 is player, pet, partyX, partypetX, raidX, raidpetX
			end
		
		
--			if ( not W_kIsRaid() ) then -- if we are not in a raid then allow player, pet, partyx partypetx to pass
--				W_Handle_Health_Event( arg1 ); --arg1 is player, pet, partyX, partypetX, raidX, raidpetX
--			else -- we are in a raid, only allow raid1-40 or raidpet 1-40 to pass
--				if ( arg1 ~= "player" and arg1 does not contain the word party ) then
--					W_Handle_Health_Event( arg1 ); --arg1 is player, pet, partyX, partypetX, raidX, raidpetX
--				end
--			end
		elseif ( event == "UNIT_MANA" and arg1 ~= "target" and arg1 ~= "mouseover" and arg1 ~="focus"  and arg1 ~= "npc" and arg1 ~= "NPC" ) then
			W_Handle_Unit_Mana_Event(); --update the total mana for player, party or raid, this function is in WhammySummary.lua
			
		end
	end
	
	
--	if ( event == "PET_UI_UPDATE" or event == "PLAYER_PET_CHANGED" or event == "PET_BAR_CHANGED"or event == "UNIT_PET") then
--		W_ShowMessage( event.."  "..arg1 );
--	end
end

--------
--------  EVENT HANDLERS --------
--------

-- Handle changes to the party or raid
function W_Handle_Addon_Loaded_Event()
	local frame;
		
	if ( W_gIsVisible == nil ) then --if this var has not been initted, then init it
		W_gIsVisible = true; --determines is the Whammy frame is visible or not
	end
	
	if ( W_gSummaryIsVisible == nil ) then
		W_gSummaryIsVisible = true;
	end

	if (W_gSavedSpells == nil) then --if no saved spells were loaded. These are the items the user has selected in the dropdowns
		W_Set_Saved_Spells(); --will set spell table defaults
	end
	
	if ( W_gSavedMacros == nil ) then --if the user has selected a macro in a drop down, this is where it is saved to
		W_gSavedMacros = {};
	end
	
	if ( W_gSavedCustomList == nil ) then --if the user has selected a custom list item, this is where is is saved to
		W_gSavedCustomList = {}; --the custom list - first item is Custom List, from there on its tables of thingName, thingType (item, spell or macro)
	end
	
	if ( W_gCustomList == nil ) then --this is where custom list items are stored
		W_gCustomList = {};
	end
	
	W_Add_Drop_Down_Options();
	
	if ( W_gHealthLevel == nil ) then
		W_gHealthLevel = "Off"; --off by default
	end
	

	
	W_Create_Unit_Buttons(); --create all of the unit buttons and also creates all of the potential pet buttons
	
	if ( W_gIsVisible ) then --if the last time the player played, it was visible, keep it visible
		frame = WhammyFrame;
		if ( frame ) then
			frame:Show();
		end
	end
	
	-- show the summary frame or not
	WhammyOptionsFrame_Summary_CheckButton:SetChecked( W_gSummaryIsVisible );
	W_Summary_CheckButton_Click( W_gSummaryIsVisible );
	
end

-- called after the spell tables have either been loaded or defaulted, to add any addtional button options: Trade, Whisper, Target etc
function W_Add_Drop_Down_Options()
	local charClass = W_GetCharClass("player");
	local tableName = ("W_"..charClass.."SpellTable"); --figure out which table to use based on char type
	local t = getglobal(tableName); -- a reference to the table
	table.insert(t, W_kMenuTrade);
	table.insert(t, W_kMenuWhisper);
	table.insert(t, W_kMenuTarget);
	table.insert(t, W_kMenuRunMacro);
	table.insert(t, W_kMenuCustomList);
end


-- called when the party members change (add, leave) - will only be called for solo and party events
function W_Handle_Party_Members_Changed_Event()
	--W_ShowMessage("Party members changed event");
	local i, numberOfPartyMembers;
	local unitID, petID, buttonName;
	local numberOfPartyMembers = GetNumPartyMembers(); -- is there anyone else in the party? excluding the player ( buttons2-5)
	
	W_ResetAllButtons();
	
	W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player

	if ( W_CharHasAPet("player") ) then
		--W_ShowMessage("  its a hunter");
		W_Set_Unit_Button_Name( "W_Pet_Button1", "pet" ); --set the button to this pet and adds the spells
	end
					
	if ( numberOfPartyMembers == 0 ) then -- the player is by themself, reset the other buttons and frame size
		W_Set_Whammy_Frame_Size( W_kPlayer );
	else -- player is in a party, populate the other player buttons and set the frame size
		for i = 1,4 do --1 through 4 are the other party members i.e. party1 is the first other party member etc
			unitID = "party"..i;
			if UnitExists( unitID ) then --if this is a valid unit then
				buttonName = "W_Unit_Button"..(i+1); --set the button to this unit
				W_Set_Unit_Button_Name( buttonName, unitID );
								
				if ( W_CharHasAPet( unitID ) ) then
					petID = "partypet"..i
					buttonName = "W_Pet_Button"..(i+1);
					W_Set_Unit_Button_Name( buttonName, petID ); --set the button to this pet and adds the spells
				end
			end
		end
		W_Set_Whammy_Frame_Size( W_kParty );
	end
end

--called when the raid roster changes or upon player login if they are in a raid
function W_Handle_Raid_Roster_Update_Event()
	--W_ShowMessage("Raid Roster Update");
	local name, rank, subgroup, level, class, fileName, zone, online, isDead;
	local unitCount, pos, id;
	local unitButtonName, unitID, petButtonName, petID;
	local subGroupUnitCount = {0,0,0,0,0,0,0,0}; --keep track of how many raid members are in each raid group. Starts at 0 (5 per group)
	W_gUnitButtonTable = {}; --clear out the global lookup table that links player unitID'S to the unit buttons they are in,
	W_gPetButtonTable = {};
	
	local numRaidMembers = GetNumRaidMembers();
	
	W_ResetAllButtons();

	if ( numRaidMembers == 0 ) then -- if there are no raid members, reset size etc to just the player
		W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player

		if ( W_CharHasAPet("player") ) then
			W_Set_Unit_Button_Name( "W_Pet_Button1", "pet" ); --set the button to this pet and adds the spells
		end
		W_Set_Whammy_Frame_Size( W_kPlayer ); --set the frame size to just player
	else
		W_Set_Whammy_Frame_Size( W_kRaid );
		for id = 1,numRaidMembers do --walk through all the raid members
			name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(id);	--get their name and subgroup
			subGroupUnitCount[ subgroup ] = subGroupUnitCount[ subgroup ] + 1; --keep track of how many units are in each subgroup
	
			unitCount = subGroupUnitCount[ subgroup ]; --get the next unit position based on the unit count of the subgroup
			
			pos = ( ( (subgroup-1) * 5) + unitCount ) -- first subgroup, first position will be 1, 8th subgroup, first position will be 36
			
			unitButtonName = "W_Unit_Button"..pos
			unitID = "raid"..id
			W_Set_Unit_Button_Name( unitButtonName, unitID );
			W_gUnitButtonTable[ unitID ] = unitButtonName; --add the buttons that are occupied to the table so we can look them up later
			if ( W_CharHasAPet( unitID ) ) then
				petButtonName = "W_Pet_Button"..pos;
				petID = "raidpet"..id
				W_Set_Unit_Button_Name( petButtonName, petID ); --set the button to this petID and adds the spells
				W_gPetButtonTable[ petID ] = petButtonName; -- store this in the pet button table so raidpet1 = W_Pet_Button1 etc
			end
		end
	end
end

-- handle unit health events for player or someone in the players party (1-4) or raid (1-40)
-- if in a raid, this will be called twice for player and party members. once as a player, again as a raid member and
--    for party members, once for the party and again for the raid
function W_Handle_Health_Event( unitID )
	--W_ShowMessage("Handle Unit Health Event for "..unitID);
	local myButtonName;
	local online, isDead;
	local partyMemberNum;
	local myTextFrame;
	local healthBarAmount;
	local isAUnit = W_IsUnit( unitID ); -- is this a unit as opposed to a pet
	local healthTargetPercent;
	local maxHealth;
	local currentHealth;
	local healthFraction;
	
	myButtonName = W_Get_Button_Name_From_UnitID( unitID ); --returns W_Unit_ButtonX or W_Pet_ButtonX
	
	if ( myButtonName ~= nil ) then
		online = UnitIsConnected( unitID );
		isDead = W_IsDead( unitID );
		
		if ( isAUnit ) then -- this is a unit and not a pet - adjust the color of the text part of the frame
			myTextFrame = getglobal( myButtonName.."_UnitName" );
			if ( online ) then
				if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
					myTextFrame:SetFontObject( W_kFontNamePlayer );
				else
					myTextFrame:SetFontObject( W_kFontNameOthers );
				end
			else --offline, show their text disabled
				myTextFrame:SetFontObject( W_kFontNameDisable );
			end
		end
		
		if ( isDead ) then --dead so show no health and adjust button texture
			W_Set_Unit_Button_Texture( myButtonName, W_kUnitDead ); --unit or pet is dead, grey out the unit button texture
			W_Set_Unit_Health_Bar_Value( myButtonName, 1); --set the healthbar to 1 (0 doesnt work)
		else -- unit or pet is alive, update health bar
			maxHealth = UnitHealthMax(unitID);
			currentHealth = UnitHealth(unitID);
			healthFraction = currentHealth / maxHealth;
			
			if ( healthFraction > 1 ) then --needed to add this becuase there is a bug in reading the pet health. Hopefully Blizz will fix in future patch
				healthFraction = 1;
			end
			
			if ( isAUnit ) then --if its a unit
				healthBarAmount = W_kHealthBarWidth * healthFraction;
			else --its a pet, use the pet healthbar width
				healthBarAmount = W_kPetHealthBarWidth * healthFraction;
			end

			W_Set_Unit_Button_Texture( myButtonName, W_kUnitAlive);
			W_Set_Unit_Health_Bar_Value( myButtonName, healthBarAmount );  -- does this line throw an error in some cases? A warlock with a pet?
			
			if W_gHealthLevel ~= "Off" then --if the user wants to be notified for low health, let them know
				--if the unit has low health, make the texture of the unit button red to get attention
				healthTargetPercent = tonumber(W_gHealthLevel)/100; --health level is the % health selected in the drop down
				if ( healthFraction < healthTargetPercent ) then -- w_gHealthLevel is a saved var
					W_Set_Unit_Button_Texture( myButtonName, W_kUnitHealthAlert )
				else --good health, set to default color
					W_Set_Unit_Button_Texture( myButtonName, W_kUnitHealthOK )
				end
			end
			
		end
	end
end

function W_IsAPet( unitID )
	return false;
end

function W_IsDead( unitID )
	local isDead = (   UnitIsDeadOrGhost( unitID ) or ( UnitHealth( unitID ) <= 0 ) or UnitIsCorpse( unitID ) or UnitIsDead( unitID )   )
	
	return isDead
end


function W_Set_Unit_Button_Texture( buttonName, whichTexture ) --set the texture of the entire unit button, either W_Unit_Button1 or W_Pet_Button1
	--W_ShowMessage("Set Unit Button Texture for unit "..unitID.." texture is "..whichTexture );
	local r, g, b, a;
	local isAUnit = getglobal( buttonName ):GetText() ~= nil --pets buttons have no name so this will be nil for pets and not nil for units
	local buttonTextureName
	
	if ( buttonName ~= nil) then
		if ( isAUnit ) then --if this is a unit do unit colors
			if ( whichTexture == W_kUnitAlive ) then
				r = 1; g = 1; b = 1; a = .2;	
				
			elseif ( whichTexture == W_kUnitDead ) then
				r = .1; g = .1; b = .1; a = .8;
				
			elseif ( whichTexture == W_kUnitHealthAlert ) then
				r = 1; g = 0; b = 0; a = .5;
				
			elseif ( whichTexture == W_kUnitHealthOK ) then
				r = 1; g = 1; b = 1; a = .2;
				
			elseif ( whichTexture == W_kUnitDisconnected ) then
				r = 0; g = 0; b = 0; a = 0;	
				
			else
				r = 1; g = 1; b = 1; a = .2;
			end
		else --must be a pet, so do pet colors
			--W_ShowMessage("   setting pet texture for button "..buttonName.." texture is "..whichTexture);
			if ( whichTexture == W_kUnitAlive ) then
				r = .2; g = .2; b = .2; a = 1;	

			elseif ( whichTexture == W_kUnitDead ) then
				r = .1; g = .1; b = .1; a = 1;
				
			elseif ( whichTexture == W_kUnitHealthAlert ) then
				r = 1; g = 0; b = 0; a = 1;
				
			elseif ( whichTexture == W_kUnitHealthOK ) then
				r = .2; g = .2; b = .2; a = 1;
				
			elseif ( whichTexture == W_kUnitDisconnected ) then
				r = 0; g = 0; b = 0; a = 0;	
				
			else
				r = .2; g = .2; b = .2; a = 1;
			end
		end
		
		buttonTextureName = buttonName.."_Texture";
		--W_ShowMessage("       buttonTextureName="..buttonTextureName)
		getglobal( buttonTextureName ):SetTexture( r, g, b, a );
	end
end

function W_Set_Unit_Health_Bar_Value( buttonName, healthBarValue ) --set the level of the health bar on the unit button
	local buttonHealthBarName = ( buttonName.."_HealthBar" ); --so we have W_Unit_Button1_HealthBar
	getglobal( buttonHealthBarName ):SetWidth( healthBarValue ); --adjust the width of the healthbar
	
	W_Update_Total_Health(); -- update the total health bar for the player, party or raid
end

function W_Set_Unit_Button_Curse_Texture( buttonName, isDebuffed, debuffType) --set the texture of the curse area on the unit button
	-- type will be Magic, Disease, Poison or Curse or nil is none
	local bTop = getglobal( buttonName.."_Curse_Texture" );
	local bLeft = getglobal( buttonName.."_CurseLeft_Texture")
	local bRight = getglobal( buttonName.."_CurseRight_Texture" );
	local bMagicText = getglobal( buttonName.."_Curse_TextM" );
	local bCurseText = getglobal( buttonName.."_Curse_TextC" );
	local bDiseaseText = getglobal( buttonName.."_Curse_TextD" );
	local bPoisonText = getglobal( buttonName.."_Curse_TextP" );

	if ( isDebuffed ) then --there is a debuff, so set the texture. Make sure to maintain
		bTop:SetTexture( .97, 1, .38, 1 ); --yellow
		bLeft:SetTexture( .97, 1, .38, 1 ); --yellow
		bRight:SetTexture( .97, 1, .38, 1 ); --yellow
		
		if (debuffType == "Magic") then --if it was magic then set the magic text otherwise clear it
			bMagicText:SetText("m");
		end
		
		if (debuffType == "Curse") then
			bCurseText:SetText("c");
		end	
			
		if (debuffType == "Disease") then
			bDiseaseText:SetText("d");
		end
			
		if (debuffType == "Poison") then
			bPoisonText:SetText("p");
		end

	else --no buff, remove the texture and set the unit name back
		bTop:SetTexture( nil );
		bLeft:SetTexture( nil );
		bRight:SetTexture( nil );
		
		bMagicText:SetText(nil); --if there are no debuffs, clear them all
		bCurseText:SetText(nil);
		bDiseaseText:SetText(nil);
		bPoisonText:SetText(nil);
	end
end

-- called when the player/pet, a party/partypet or a raid member/raid pet  buffs/debuffs change.
function W_Handle_Aura_Event( unitID )
	--W_ShowMessage( "Unit Aura Event for unitid "..unitID);
	local debuffName, debuffRank, debuffTexture, debuffApplications, debuffType;
	local debuffNum = 1;
	local showDispellable = 1; -- 1 = only return debuffs that are dispellable, 0 = all buffs
	local foundOne, unitButtonName;

	unitButtonName = W_Get_Button_Name_From_UnitID( unitID ); --returns W_Unit_ButtonX or W_Pet_ButtonX
	
	if ( unitButtonName ~= nil) then
		W_Set_Unit_Button_Curse_Texture( unitButtonName, false, nil); --start by clearing the curse bar and letters
		while ( UnitDebuff( unitID, debuffNum) ) do -- while the unit has debuffs, walk through them as see if there are any we can fix
			debuffName, debuffRank, debuffTexture, debuffApplications, debuffType = UnitDebuff( unitID, debuffNum, showDispellable );
			if (debuffTexture ~= nil) then --for each one found, pass the type which will then display the corresponding letter M, D, P or C
				W_Set_Unit_Button_Curse_Texture( unitButtonName, true, debuffType); --type will be Magic, Disease, Poison or Curse
			end
			debuffNum = debuffNum + 1;
		end
	end
end

--called when health or aura events occur in a raid
--   determine which button is associated with the passed in unitID. Pass in player, pet, party1-4, partypet 1-4, raid 1-40 or raidpet1-40
--   return the button name that refers to this unit or pet either W_Unit_ButtonX or W_Pet_ButtonX
function W_Get_Button_Name_From_UnitID( unitID )
	local b;
	local id;
	
	--W_ShowMessage( "Getting Button name for unit "..unitID );
	
	if ( unitID == "player" ) then
		b = "W_Unit_Button1"
	elseif ( unitID == "pet" ) then
		b = "W_Pet_Button1"
	else --we are either in a party or a raid, and could be a unit or a pet
		if ( W_IsRaid() ) then
			if ( strfind(arg1, "raid" ) ) then --does the unitID contain the text raid, then let it pass
				b = W_Get_Raid_Unit_Button_From_UnitID( unitID ); --look up which button this raid unit is in via the global table
			end
		else --its a party
			if ( W_IsUnit( unitID ) ) then --its a player unit
				id, numberFound = gsub( unitID, "party([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
				b = "W_Unit_Button"..(id+1);
			else --its a pet
				id, numberFound = gsub( unitID, "partypet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
				b = "W_Pet_Button"..(id+1)
			end
		end
	end

	return b
end

-- called when we enter or leave combat. Enable or disable the drop downs depending on combat status
function W_Handle_Combat( isInCombat )
	local dropDownName;
	
	if ( isInCombat ) then
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			dropDownName = "W_DropDown"..i
			UIDropDownMenu_DisableDropDown( getglobal( dropDownName ) );
		end
	else
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			dropDownName = "W_DropDown"..i
			UIDropDownMenu_EnableDropDown( getglobal( dropDownName ) );
		end
	end
end
--
--------  END EVENT HANDLERS --------
--


--
-- functions called to set up the unit buttons to respond to mouse clicks
--
-- called from the handle raid roster update and party change events to assign unit names to unit buttons
function W_Set_Unit_Button_Name( buttonName, unitID)
	local newStringWidth;
	local b, unitName, buttonWidth, stringWidth;
	--W_ShowMessage( "Setting Unit Button "..buttonName.." to "..unitID);
	
	getglobal(buttonName):SetAttribute("unit", unitID); --set the assignment of the button to the passed in unit  (need to do this first)
	
	if ( W_IsUnit( unitID ) ) then --if it is a unit instead of a pet, put the units name in the button
		b = getglobal( buttonName.."_UnitName" );
		unitName = UnitName(unitID); -- unit ID will be "player" or "party1" or "raid1", and this will give us the actual name
		b:SetText(unitName); --set the text of the passed in button to the unit name
		
		buttonWidth = getglobal(buttonName):GetWidth();
		
		stringWidth = b:GetStringWidth();
		
		if ( stringWidth > buttonWidth ) then
			newStringWidth = buttonWidth * .90 -- the new string width will be 80% of the button width
			b:SetWidth( newStringWidth );
		else --the unit name fit within the box so reset the string width value
			b:SetWidth( buttonWidth );
		end
		
		if ( UnitIsUnit( unitID, "player" ) ) then -- if this button is the player button, make it stand out so it is more easily picked out
			b:SetFontObject( W_kFontNamePlayer );
		else
			if ( UnitIsConnected( unitID ) ) then --if the unit is connected, standard font color
				b:SetFontObject( W_kFontNameOthers );
			else --if the unit disconnects then change the font color to disconnected color
				b:SetFontObject( W_kFontNameDisable );
			end
		end
	else --it was a pet, make sure the button is visible
		getglobal(buttonName):Show()
	end
	
	W_Set_Unit_Button_Click( buttonName );
end

--walk through and update all of the unit buttons to reflect a change in spell or unit etc
function W_Update_Unit_Buttons() 
	for i = 1,W_kMaxUnitButtons do --now that we changed a spell, walk through and update all of the unit buttons to reflect the change
		W_Set_Unit_Button_Click( "W_Unit_Button"..i );
		W_Set_Unit_Button_Click( "W_Pet_Button"..i);
	end
end

--called from handle raid roster update, dropdown selection change events and default button click.
--      Assign mouse clicks to the passed in unit button
function W_Set_Unit_Button_Click( buttonName )
	--W_ShowMessage("setting ".. unitButtonName);
	W_Set_Click("", "1", buttonName, "W_DropDown1" ); --Left Click is DropDown1
	W_Set_Click("", "2", buttonName, "W_DropDown3" ); -- Right Click is DropDown3
	W_Set_Click("", "3", buttonName, "W_DropDown2" ); --Middle Button Click is DropDown2
	W_Set_Click("ctrl", "1", buttonName, "W_DropDown4" ); --Ctrl Left
	W_Set_Click("ctrl", "2", buttonName, "W_DropDown6" ); --Ctrl Right
	W_Set_Click("ctrl", "3", buttonName, "W_DropDown5" ); --Ctrl Middle
	W_Set_Click("shift", "1", buttonName, "W_DropDown7" ); --Shift Left
	W_Set_Click("shift", "2", buttonName, "W_DropDown9" ); --Shift Right
	W_Set_Click("shift", "3", buttonName, "W_DropDown8" ); --Shift Middle
	W_Set_Click("alt", "1", buttonName, "W_DropDown10" ); --Alt Left
	W_Set_Click("alt", "2", buttonName, "W_DropDown12" ); --Alt Right
	W_Set_Click("alt", "3", buttonName, "W_DropDown11" ); --Alt Middle
end

-- Called from W_Set_Unit_Button_Click to set what the mouse click does in reponse to the players frameButton
function W_Set_Click(keyPress, mouseButton, buttonName, ddMenuName )
	--keyPress will be filled if there is a keyboard button pressed along with the mouse click
	--mouseButton is: 1=Left, 2=Right, 3=Middle, 4 and 5
	--buttonName is the frame button on the screen we are assigning the clicks to
	--ddMenuName is the dropdown menu name this spell is saved for
	
	local myUnitID, myUnitName, myButton, menuItem, itemName;
	local chatType, channel;
	local thingTable, thingName, thingType;
	
	myButton = getglobal( buttonName ); --get a pointer to the unitButton that we are configuring
	
	if ( myButton:GetAttribute("unit") ~= nil ) then -- the unit attrib would be nil if it is does not have a unit assigned to it
		if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
			keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
		end
		--get the item from the saved spell table: spell, Whisper, Trade, W_kMenuRunMacro, W_kMenuCustomList
		menuItem, itemName = W_Get_Saved_MenuItem( ddMenuName );
		
		if ( menuItem == W_kMenuTrade ) then --this button will be set to trade
			myUnitID = myButton:GetAttribute("unit");
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Trade");
			myButton.W_Perform_Trade = function() InitiateTrade(myUnitID); W_ShowMessage(W_kMsgOpeningTradeWindow); end;
			
		elseif ( menuItem == W_kMenuWhisper ) then
			myUnitID = myButton:GetAttribute("unit");
			myUnitName = UnitName( myUnitID );
			chatType = "WHISPER";
			channel = GetChannelName( chatType );
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Chat_Message");
			myButton.W_Perform_Chat_Message = function() ChatFrame_SendTell(myUnitName); W_ShowMessage(W_kMsgWhisper); end;
			
		elseif ( menuItem == W_kMenuTarget ) then --just target the unit
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "target");
			
		elseif ( menuItem == W_kMenuRunMacro) then
			--W_ShowMessage( "W_SetClick detected a macro"..macroName );
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "macro");
			myButton:SetAttribute( (keyPress.."macro"..mouseButton), itemName); --which spell is actually a macro here.
			
		elseif ( menuItem == W_kMenuCustomList ) then --it's an item from the custom list
			thingTable = itemName; --in this case the itemName will be a table of itemName and type
			thingName = thingTable[1];
			thingType = thingTable[2];

			myButton:SetAttribute( (keyPress.."type"..mouseButton), thingType); --thingType is item, spell or macro
			myButton:SetAttribute( (keyPress..thingType..mouseButton), thingName);

		else --we are setting the button to a spell - either one from the list or the custom list
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "spell");
			myButton:SetAttribute( (keyPress.."spell"..mouseButton), menuItem);
		end
	end
end

function W_Open_Trade_Macro()
	InitiateTrade("target");
	--W_ShowMessage("opening trade window");
end

function W_ResetAllButtons()
	local unitButtonName, petButtonName;
	local ubTextFrame;
	local ub, pb;


	for i = 1,W_kMaxUnitButtons do --clear out the raid buttons
		unitButtonName = "W_Unit_Button"..i
		petButtonName = "W_Pet_Button"..i
		ubTextFrame = getglobal( unitButtonName.."_UnitName" );
		ub = getglobal( unitButtonName );
		pb = getglobal( petButtonName);
		
		-- set the text of the button to -
		ubTextFrame:SetText("-");
		ubTextFrame:SetFontObject( W_kFontNameOthers );
		
		-- set the action of the button to nil
		ub:SetAttribute("type", "stop"); --set the function of the button to nothing
		ub:SetAttribute("unit", nil); --set the unit attrib (the unit name) to nothing
		
		-- set the color of the button to standard
		W_Set_Unit_Button_Texture( unitButtonName, W_kUnitHealthOK );
		
		-- set the curse of the button to off
		W_Set_Unit_Button_Curse_Texture( unitButtonName, false, nil);
		
		-- set the health bar to full health
		W_Set_Unit_Health_Bar_Value( unitButtonName, W_kHealthBarWidth);
		
		
		pb:SetAttribute("type", "stop"); --set the function of the button to nothing
		pb:SetAttribute("unit", nil); --set the unit attrib (the unit name) to nothing
		pb:Hide();
		
		-- set the color of the button to standard
		W_Set_Unit_Button_Texture( petButtonName, W_kUnitHealthOK );
		
		-- set the curse of the button to off
		W_Set_Unit_Button_Curse_Texture( petButtonName, false, nil);
		
		-- set the health bar to full health
		W_Set_Unit_Health_Bar_Value( petButtonName, W_kPetHealthBarWidth);
	end
end

--
-- W_DropDownMenu stuff: the onload function defines which function init's the button (which is automatically called)
--
-- load up the action buttons with default spells if there were no saved spells
function W_Set_Saved_Spells()
	local dropDownButtonName;
	
	W_ShowMessage(W_kMsgSettingSpellsToDefault);
	if ( W_gSavedSpells == nil ) then
		W_gSavedSpells = {}; --create the table
	end
	
	local charClass = W_GetCharClass("player"); --what class of player is this: used to populate Hunter pet buttons
	local defaultTableName = "W_"..charClass.."DefaultSpellTable"; -- the default spell table for this class ie W_PriestDefaultSpellTable

	for i = 1,W_kMaxDropDownButtons do --for each drop down button, get what it's default should be from the default tables for this char type
		dropDownButtonName = "W_DropDown"..i;
		W_gSavedSpells[ dropDownButtonName ] = getglobal(defaultTableName)[i]; -- this is W_DropDown1="Greater Heal" and W_DropDown2="Renew" etc
	end
end

--	use for troubleshooting
function W_Show_Spells() -- this function is for troubleshooting
	local value;
	
	for k, v in pairs(  W_gSavedSpells ) do --iterate over the key, values pairs from the spell table appropriate for this char
		value = W_gSavedSpells[k];
	end
end


--
-- MISC functions from here down
--

-- Display a message in the chat frame
function W_ShowMessage(aMessage)
	local r,g,b,a
	r = 0;
	g = 1;
	b = 0;
	a = .5;
	
	if ( aMessage ~= nil ) then
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: " ..aMessage, r, g, b, a);
	else
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: nil message");
	end
end



function W_Options_Button_Click()
	WhammyOptionsFrame:Show();
end

function W_Test_Button_Click()
	W_ShowMessage("test button");
end

function W_Help_Test_Button_Click()
	W_ShowMessage("help test button");
end

function W_Info_Button_Click() -- show/hide the info frame when the user clicks the ? button
	local frame = WhammyInfoFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	end
end

--
-- When the info window loads, set the text that should appear
--
function W_Info_Onload(self)
	--W_ShowMessage("info onload");
	local infoText
	local kHelpFrameWidth = 400;
	
	WhammyInfoFrame:SetWidth( kHelpFrameWidth ); --adjust the size of the frame
	WhammyInfoFrame_Heading:SetText("Whammy "..WhammyVers);
	
	-- note that the height of the info frame will adjust around the height of the text. Did this for localization purposes in case another language takes
	--    more or less space to say the same thing
	local t = WhammyInfoFrame_Help_Text;
	
	t:SetPoint("TOPLEFT", "WhammyInfoFrame", 10, -35); --set the starting position of the help text
	t:SetWidth( kHelpFrameWidth - 8); --set the width of the text
	t:SetText( W_kHelpText ); --set the text from the static text
	local textHeight = t:GetHeight(); --get the height of the text so we can adjust the frame height around it
	WhammyInfoFrame:SetHeight( textHeight + 40 ) --set the frame height to the height of the text + space for the heading area
end


-- called to adjust the whammy frame size depending on single player, party or raid size
function W_Set_Whammy_Frame_Size( aGroupType )
	local newWidth, newHeight, numToShow;
	local marginSides = 29;
	local marginTopBottomParty = 38;
	local marginTopBottomRaid = 88;
	local buttonGutter = 3;
	local buttonName;
	--aGroupType = W_kParty;

	if ( aGroupType == W_kPlayer ) then
		newWidth = ( W_kUnitButtonWidth + marginSides );
		newHeight = ( W_kUnitButtonHeight + 40 );
		numToShow = 1; --1 is default size for single player
	elseif ( aGroupType == W_kParty ) then
		newWidth = ( W_kUnitButtonWidth + buttonGutter ) + marginSides;
		newHeight = ( (W_kUnitButtonHeight + buttonGutter) * 5) + marginTopBottomParty;
		numToShow = 5;
	elseif ( aGroupType == W_kRaid ) then
		newWidth = ( (W_kUnitButtonWidth + buttonGutter) * 4) + marginSides;
		newHeight = ( (W_kUnitButtonHeight + buttonGutter) * 8) + marginTopBottomRaid;
		numToShow = 40;
	else
		newWidth = 100
		newHeight = 80
		numToShow = 1;
	end
	
	for i = 1,numToShow do --show the unit buttons that need to be seen
		buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Show();
	end
	
	for i = numToShow+1,40 do --hide the unused unit buttons
		buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Hide();
		buttonName = "W_Pet_Button"..i
		getglobal( buttonName):Hide()
	end
	
	WhammyFrame:SetWidth( newWidth ); --adjust the size of the frame
	WhammyFrame:SetHeight( newHeight );
end

-- create all of the potential unit buttons. They start off hidden except for #1, which by default is the player
function W_Create_Unit_Buttons()
	for i = 1,40 do
		W_Create_Button(i);
	end
	
	W_Set_Button_Orientation( W_kOrientRect );
	--W_Set_Button_Orientation( W_kOrientHorizontal );
end

function W_Create_Button( buttonNum )
	local buttonFrameName = "W_Unit_Button"..buttonNum;

	--create the frame: pass the  type, name, child of, template
	local f = CreateFrame("Button", buttonFrameName, WhammyFrame, "WhammyUnitButtonTemplate");

	f:SetWidth( W_kUnitButtonWidth ); --set the default width and height for each button
	f:SetHeight( W_kUnitButtonHeight );
	f:SetID(buttonNum);
	f:Hide(); --start off hidden
	
	getglobal( buttonFrameName.."_HealthBar" ):SetWidth( W_kHealthBarWidth )
	getglobal( buttonFrameName.."_HealthBar" ):SetHeight( W_kHealthBarHeight )
	
	getglobal( buttonFrameName.."_Curse" ):SetWidth( W_kCurseWidth );
	getglobal( buttonFrameName.."_Curse" ):SetHeight( W_kCurseHeight )
	
	getglobal( buttonFrameName.."_CurseLeft"):SetWidth( W_kCurseLeftSideWidth );
	getglobal( buttonFrameName.."_CurseLeft"):SetHeight( W_kCurseSidesHeight );

	getglobal( buttonFrameName.."_CurseRight"):SetWidth( W_kCurseRightSideWidth );
	getglobal( buttonFrameName.."_CurseRight"):SetHeight( W_kCurseSidesHeight );
	
	W_Create_PetButton( buttonNum, buttonFrameName ); -- create a potential pet button beside each unit button
end

function W_Set_Button_Orientation( whichOrientation )
	local buttonWidth = W_kUnitButtonWidth;
	local buttonHeight = W_kUnitButtonHeight;
	local buttonGutter = 3;
	local leftMargin = 15;
	local topMargin = 28;
	local buttonName, f, x, y;
	
	
	if ( whichOrientation == W_kOrientRect ) then --the standard is a rectangle
		local buttonNum = 1;
		for half = 1,2 do --the standard whammy frame has a top half, groups 1-4, and a bottom half, groups 5-8
			for col = 1,4 do
				for row = 1,5 do
					buttonName = "W_Unit_Button"..buttonNum;
					f = getglobal( buttonName );
					x = leftMargin + ( (col-1) * (buttonWidth + buttonGutter) );
					y = topMargin + ( (row-1) * (buttonHeight + buttonGutter) );
					f:SetPoint("TOPLEFT", "WhammyFrame", "TOPLEFT", x, -y);
					buttonNum = buttonNum + 1;
				end
			end
		
		topMargin = topMargin + 118;
		end
	elseif ( whichOrientation == W_kOrientHorizontal ) then
		y = topMargin;
		buttonWidth = 70;
		for col = 1,40 do
			buttonName = "W_Unit_Button"..col;
			f = getglobal( buttonName );
			x = leftMargin + ( (col-1) * (buttonWidth + buttonGutter) );
			f:SetPoint("TOPLEFT", "WhammyFrame", "TOPLEFT", x, -y);
		end
	end
end
-- End Unit button creation


-- toggle the config frame shown and hidden
function W_Config_Frame_Toggle()
	local configFrame = WhammyConfigFrame;
	local wFrame = WhammyFrame;
	local parentFrame = UIParent; --need to make sure we open the config frame in the work area
	local parentFrameWidth = parentFrame:GetWidth(); -- get the width of the main window
	local configFrameWidth = configFrame:GetWidth();
	local wFrameLeft = floor( wFrame:GetLeft() + 0.5); --get left and make it in integer
	local wFrameTop = floor( wFrame:GetTop() + 0.5);
	local wFrameWidth = floor( wFrame:GetWidth() + 0.5);
	
	local totalWidth = wFrameLeft + wFrameWidth + configFrameWidth
			
	if ( configFrame ) then
		if ( configFrame:IsVisible() ) then --hide the frame if it's visible
			configFrame:Hide();
		else --show the frame if it's hidden

			configFrame:ClearAllPoints(); --releases the frame so it can be repositioned
							
			if ( totalWidth > parentFrameWidth ) then --if the config window is off the right side of the screen, open on the left of Whammy
				--obj:SetPoint(point, frame, relativePoint , x, y);
				configFrame:SetPoint("TOPRIGHT", wFrame, "TOPLEFT", -5, -5)
			else -- it was not off the right side of the screen, so open it on the right
				configFrame:SetPoint("TOPLEFT", wFrame, "TOPRIGHT", 5, -5)
			end
			
			configFrame:Show();
		end
	end
end



-- handle our one and only slash command
function W_Slash_Command_Handler(msg)
	--W_ShowMessage("slash handler");
	local frame = WhammyFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
			W_gIsVisible = false;
			if ( WhammyConfigFrame:IsVisible() ) then
				WhammyConfigFrame:Hide();
			end
		else
			frame:Show();
			W_gIsVisible = true;
		end
	end
end

-- get an id from the unitID - pass in either party1-party4 or partypet1-partypet4 or raid1-raid40 or raidpet1-raidpet40
function W_GetID( unitID ) --unit type will be either unit or pet
	local id, numberFound;
	
	if ( W_IsUnit( unitID ) ) then -- make sure this is unit, not a pet
		if ( GetNumRaidMembers() == 0 ) then -- party
			id, numberFound = gsub( unitID, "party([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		else
			id, numberFound = gsub( unitID, "raid([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		end
	else -- it was a pet, get it's id
		if ( GetNumRaidMembers() == 0 ) then
			id, numberFound = gsub( unitID, "partypet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		else
			id, numberFound = gsub( unitID, "raidpet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		end
	end
	
	if ( numberFound > 0 ) then
		return id;
	else
		return nil;
	end
end

function W_IsUnit( unitID )
	if ( string.find( unitID, "pet") == nil ) then --look for pet in the unitID, which could be pet, partypet or raidpet
		return true
	else
		return false
	end
end

function W_IsPet( unitID )
	if ( string.find( unitID, "partypet") ~= nil ) then
		return true
	else
		return false
	end
end

function W_IsRaid()
	local numRaidMembers = GetNumRaidMembers();
	
	if ( numRaidMembers == 0 ) then
		return false;
	else
		return true;
	end
end

-- for a raid: return the unit button the unit is in, or the unit button the raid p
function W_Get_Raid_Unit_Button_From_UnitID( unitID ) --this is passed raid1-40, or raidpet1-40
	local buttonName;
	
	if ( W_IsUnit( unitID ) ) then
		buttonName = W_gUnitButtonTable[ unitID ]; --this table is filled in whenever there is a raid roster update
	else
		buttonName = W_gPetButtonTable[ unitID ];
	end

	return buttonName
end

--retrieve a saved spell for the passed in drop down
function W_Get_Saved_MenuItem( ddName )
	local menuItem, thing;
	
	menuItem = W_gSavedSpells[ ddName ]; --this is the actual name of a spell, Whisper, Trade, Target, W_kMenuRunMacro or W_kMenuCustomList
	
	if (menuItem == W_kMenuRunMacro) then --it was Run Macro, retrieve the macro from the saved macros table for this dropdown
		thing = W_gSavedMacros[ ddName ]; --return the macro name
	elseif (menuItem == W_kMenuCustomList) then -- the thing was Custom List - either an item, spell or macro
		thing = W_gSavedCustomList[ ddName ]; --get the table that contains the item name and whether its and item, spell or macro
	else --it was not a macro or custom, so just return the spell name
		thing = nil;
	end
	
	return menuItem, thing --macro name will be nil if there is no menu item
end

-- get the type of char this is, and use the english version of the char type. See what the char type is for spell table loading and to see if it has a pet
function W_GetCharClass( unitID )
	local localizedClass, englishClass = UnitClass( unitID );
	return englishClass
end

-- determine if the passed in unitID is a char that has a pet
function W_CharHasAPet( unitID )
	local charType = W_GetCharClass( unitID ) --get the type of char this is in english

	if ( charType == "HUNTER" ) then
		return true;
	end
end